/*****************************************************************************
   callbacks.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/



void volume_adjust (GtkWidget *widget, CardInfo *info)
{
  GtkWidget *adj;
  GtkWidget *button;
  int err;

  adj = button = NULL;
  
  if(info->flags & CHANNEL_LEFT || 
     info->flags & CHANNEL_RIGHT ||
     info->flags & CHANNEL_MONO)
    adj = widget;

  if(info->flags & CHANNEL_MUTE_RIGHT ||
     info->flags & CHANNEL_MUTE_LEFT ||
     info->flags & CHANNEL_RECORD ||
     info->flags & CHANNEL_SIMULTANEOUS ||
     info->flags & CHANNEL_MUTE_MONO)
      button = widget;

  
  if(info->flags & CHANNEL_LEFT || info->flags & CHANNEL_RIGHT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS) {
      info->chan->data.left = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      info->chan->data.right = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      GTK_ADJUSTMENT (info->other)->value = GTK_ADJUSTMENT (adj)->value;
      gtk_signal_emit_by_name (GTK_OBJECT (info->other), "changed");
    }
    else {
      if(info->flags & CHANNEL_LEFT)
	info->chan->data.left = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      
      if(info->flags & CHANNEL_RIGHT)
	info->chan->data.right = 100 - (int)GTK_ADJUSTMENT (adj)->value;
    }

  if(info->flags & CHANNEL_MONO) 
    info->chan->data.left = 
      info->chan->data.right = 
      100 - (int)GTK_ADJUSTMENT (adj)->value;
    

  if(info->flags & CHANNEL_MUTE_RIGHT || info->flags & CHANNEL_MUTE_LEFT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS)
      if(GTK_TOGGLE_BUTTON (button)->active) {
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), TRUE);
      }
      else {
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), FALSE);
      }
    else {
      if(info->flags & CHANNEL_MUTE_RIGHT) 
	if(GTK_TOGGLE_BUTTON (button)->active)
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	else
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
      
      if(info->flags & CHANNEL_MUTE_LEFT)
	if(GTK_TOGGLE_BUTTON (button)->active)
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	else
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    }
  

  if(info->flags & CHANNEL_MUTE_MONO)
    if(GTK_TOGGLE_BUTTON (button)->active)
      info->chan->data.flags |= (SND_MIXER_FLG_MUTE_RIGHT | 
				 SND_MIXER_FLG_MUTE_LEFT | 
				 SND_MIXER_FLG_MUTE);
    else
      info->chan->data.flags &= ~(SND_MIXER_FLG_MUTE_RIGHT | 
				  SND_MIXER_FLG_MUTE_LEFT | 
				  SND_MIXER_FLG_MUTE);
  
  if(info->flags & CHANNEL_RECORD)
    if(GTK_TOGGLE_BUTTON (button)->active)
      info->chan->data.flags |= SND_MIXER_FLG_RECORD;
    else
      info->chan->data.flags &= ~SND_MIXER_FLG_RECORD;
  

  if(info->flags & CHANNEL_SIMULTANEOUS)
    if(GTK_TOGGLE_BUTTON (button)->active)
      info->chan->flags |= CHANNEL_SIMULTANEOUS;
    else
      info->chan->flags &= ~CHANNEL_SIMULTANEOUS;

  /* Write whatever we've done to the kernel */
  err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &info->chan->data);
  if(err < 0)
    printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
}

void something_changed(gpointer data, int fd, GdkInputCondition condition)
{
  int err;
  snd_mixer_callbacks_t callbacks;
  Card *this_card = (Card *)data;

  /* Set up and install the status change callback */
  bzero(&callbacks, sizeof(callbacks));
  callbacks.private_data = (void *)this_card;
  callbacks.channel_was_changed = channel_changed;
  err = snd_mixer_read(this_card->handle, &callbacks);
  if(err < 0)
    printf("Error: %s", snd_strerror(err));
}

void channel_changed (void *private_data, int channel)
{
  Card *crd = (Card *)private_data;
  int sim = 0;
  
  /* If the simultaneous button is set, undo the flag for a moment to not screw up independent actions */
  if(crd->channel[channel].flags & CHANNEL_SIMULTANEOUS) {
    sim = 1;
    crd->channel[channel].flags &= ~CHANNEL_SIMULTANEOUS;
  }

  /********************/
  /* Read in the data */
  snd_mixer_channel_read(crd->handle, channel, &crd->channel[channel].data);


  /***************************/
  /* Set the scales properly */
  /* Left */
  if(crd->channel[channel].ladj) {
    GTK_ADJUSTMENT (crd->channel[channel].ladj)->value = 
      (float) (100 - crd->channel[channel].data.left);
    gtk_signal_emit_by_name (GTK_OBJECT (crd->channel[channel].ladj), 
			     "changed");
  }
  /* Right */
  if(crd->channel[channel].ladj) {
    GTK_ADJUSTMENT (crd->channel[channel].radj)->value = 
      (float) (101 - crd->channel[channel].data.right);
    gtk_signal_emit_by_name (GTK_OBJECT (crd->channel[channel].radj), 
			     "changed");
  }

  /* Mono */
  if(crd->channel[channel].madj) {
    GTK_ADJUSTMENT (crd->channel[channel].madj)->value = 
      (float) (101 - crd->channel[channel].data.right);
    gtk_signal_emit_by_name (GTK_OBJECT (crd->channel[channel].madj), 
			     "changed");
  }

  /******************/
  /* Set the muting */
  /* Left */
  if(crd->channel[channel].lm)
    if (crd->channel[channel].data.flags & SND_MIXER_FLG_MUTE_LEFT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].lm), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].lm), 
				  FALSE);

  /* Right */
  if(crd->channel[channel].rm)
    if(crd->channel[channel].data.flags & SND_MIXER_FLG_MUTE_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].rm), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].rm), 
				  FALSE);
  /* Mono */
  if(crd->channel[channel].mm)
    if(crd->channel[channel].data.flags & SND_MIXER_FLG_MUTE || 
       crd->channel[channel].data.flags & SND_MIXER_FLG_MUTE_LEFT ||
       crd->channel[channel].data.flags & SND_MIXER_FLG_MUTE_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON
				  (crd->channel[channel].mm),
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON
				  (crd->channel[channel].mm),
				  FALSE);
  
  /******************/
  /* Set the Record */
  if(crd->channel[channel].rec)
    if(crd->channel[channel].data.flags & SND_MIXER_FLG_RECORD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].rec), 
				  TRUE);
    else
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON 
				  (crd->channel[channel].rec), 
				  FALSE);

  /* If the simultaneous button is set, reset the flag */
  if(sim)
    crd->channel[channel].flags |= CHANNEL_SIMULTANEOUS;


  return;
}
