/*****************************************************************************
   cards.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/


int init_cards()
{
  /* Begin variable declarations */
  int i,j;
  int err;
  void *handle;
  struct snd_ctl_hw_info cardinfo; /* The info for the card */
 /* Begin variable declarations */

  /* Get the number of sound cards in the system */
  cards = snd_cards();

  /* Allocate out the handles */
  card = calloc(cards, sizeof(Card));

  /* Open up the mixers and get the mixer info. */
  for(i = 0; i < cards; i++) {
    /* Open the mixer */
    err = snd_mixer_open(&(card[i].handle), i, 0);
    if(err < 0) {
      card[i].handle = NULL;
      printf("Unable to open mixer for soundcard %i.  Error: %s\n", i, 
	     snd_strerror(err) );
    }

    /* Get the mixer information */
    if(card[i].handle)
      snd_mixer_info(card[i].handle, &card[i].info);

    /* Get the sound card's info - we only need the name really */
    err = snd_ctl_open(&handle, i);
    if(err < 0) {
      handle = NULL;
      printf("Unable to open control for soundcard %i.  Error: %s\n", i, 
	     snd_strerror(err) );
      /* We'll take the mixer's name instead */
      strcpy(card[i].name, card[i].info.name);
    }
    if(handle) {
      err = snd_ctl_hw_info(handle, &cardinfo);
      if(err < 0){
	printf("Unable to get control hardware info for soundcard %i.  Error: %s\n.", i, snd_strerror(err));
	/* We'll take the mixer's name instead */
	strcpy(card[i].name, card[i].info.name);
      }
      strcpy(card[i].name,cardinfo.name);
      snd_ctl_close(handle);
    }




  }

  /* Get the number of channels */
  for(i = 0; i < cards; i++)
    card[i].cnum = snd_mixer_channels(card[i].handle);

  /* Allocate the channels out */
  for(i = 0; i < cards; i++)
    if(card[i].cnum > 0)
      card[i].channel = calloc(card[i].cnum, sizeof(Channel));
    else
      card[i].channel = NULL;

  /* Get the channel info and data */
  for(i = 0; i < cards; i++)
    for(j = 0; j < card[i].cnum; j++) {
          err = snd_mixer_channel_info(card[i].handle, 
				       j,
				       &card[i].channel[j].info);
	  err = snd_mixer_channel_read(card[i].handle, 
				       j,
				       &card[i].channel[j].data);
	  card[i].channel[j].num = j;
    }


  return 1;
}
