/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/


GtkWidget *create_volume_control(char *name, 
				 float left, 
				 float right, 
				 Card *card, 
				 int channel, 
				 snd_mixer_channel_t *data, 
				 snd_mixer_channel_info_t *chan_info, 
				 Channel *schan)
{
  GtkWidget *box, *box1, *box2;
  GtkWidget *rscale, *lscale;
  GtkObject *ladj, *radj, *madj; 
  GtkWidget *label;
  CardInfo *info;
  GtkWidget *lmute, *rmute, *mmute;
  GtkWidget *button;

  rmute = lmute = mmute = NULL;

  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box);
  
  /* Create the control label */
  label = gtk_label_new(name);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0);

  /* Make the box to hold the volume range controls and the mute buttons*/
  box1 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(box1);
  gtk_box_pack_start(GTK_BOX (box), box1, FALSE, FALSE, 0);
  
  /********************/
  /* The Widgets *****/

  if(chan_info->caps & SND_MIXER_CINFO_CAP_STEREO) {
    /*****************/
    /* The left side */
    
    /* Make the box to hold each individual range and mute button */
    box2 = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(box2);
    gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, TRUE, 0);
    
    /* Start with the left volume control */
    ladj = gtk_adjustment_new(101.0 - left, 0.0, 101.0, 1.0, 10.0, 10.0);
    lscale = gtk_vscale_new(GTK_ADJUSTMENT (ladj));
    gtk_widget_show(lscale);
    gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
    gtk_box_pack_start(GTK_BOX (box2), lscale, FALSE, FALSE, 0);
    /* Store the widget in the channel struct */
    schan->ladj = ladj;
    
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the right mute button */
      lmute = gtk_toggle_button_new_with_label("M");
      gtk_widget_show(lmute);
      gtk_box_pack_start(GTK_BOX(box2), lmute, FALSE, FALSE, 0);
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_LEFT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (lmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->lm = lmute;
    
    
    
    /******************/
    /* The right side */
    
    /* Make the box to hold each individual range and mute button */
    box2 = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(box2);
    gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, TRUE, 0);
    
    /* Create the right volume control */
    radj = gtk_adjustment_new(101.0 - right, 0.0, 101.0, 1.0, 10.0, 10.0);
    rscale = gtk_vscale_new(GTK_ADJUSTMENT (radj));
    gtk_widget_show(rscale);
    gtk_scale_set_draw_value(GTK_SCALE (rscale), FALSE);
    gtk_box_pack_start(GTK_BOX (box2), rscale, FALSE, FALSE, 0);
    /* Store the widget in the channel struct */
    schan->radj = radj;
    
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the left mute button */
      rmute = gtk_toggle_button_new_with_label("M");
      gtk_widget_show(rmute);
      gtk_box_pack_start(GTK_BOX(box2), rmute, FALSE, FALSE, 0);
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_RIGHT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (rmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->rm = rmute;
    
    
    /**********************/
    /* The Callbacks *****/
    
    /* The left */
    /* The Scale callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->other = (GtkWidget *)radj;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_LEFT;
    gtk_signal_connect(GTK_OBJECT (ladj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);
    
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* The Mute callback */
      info = calloc(1, sizeof(CardInfo));
      info->card = card;
      info->channel = channel;
      info->other = rmute;
      info->chan = schan;
      info->flags &= 0;
      info->flags |= CHANNEL_MUTE_LEFT;
      gtk_signal_connect(GTK_OBJECT (lmute), "toggled",
			 (GtkSignalFunc) volume_adjust, info);
    }
    
    
    /* The Right */
    
    /* Set up the Right callback */  
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->other = (GtkWidget *)ladj;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_RIGHT;
    gtk_signal_connect(GTK_OBJECT (radj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);
    
    /* Set up the Mute callback */
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      info = calloc(1, sizeof(CardInfo));
      info->card = card;
      info->channel = channel;
      info->other = lmute;
      info->chan = schan;
      info->flags &= 0;
      info->flags |= CHANNEL_MUTE_RIGHT;
      gtk_signal_connect(GTK_OBJECT (rmute), "toggled",
			 (GtkSignalFunc) volume_adjust, info);
    }
    
  }
  /* If the channel isn't stero */
  else {
    madj = gtk_adjustment_new(101.0 - left, 0.0, 101.0, 1.0, 10.0, 10.0);
    lscale = gtk_vscale_new(GTK_ADJUSTMENT (madj));
    gtk_widget_show(lscale);
    gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
    gtk_box_pack_start(GTK_BOX (box), lscale, FALSE, FALSE, 0);
    /* Store the widget in the channel struct */
    schan->madj = madj;

    /* Set up the volume callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->other = NULL;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_MONO;
    gtk_signal_connect(GTK_OBJECT (madj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);


    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the right mute button */
      mmute = gtk_toggle_button_new_with_label("M");
      gtk_widget_show(mmute);
      gtk_box_pack_start(GTK_BOX(box), mmute, FALSE, FALSE, 0);
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_LEFT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (mmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->mm = mmute;

    /* Set up the mute callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->other = NULL;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_MUTE_MONO;
    gtk_signal_connect(GTK_OBJECT (mmute), "toggled",
		       (GtkSignalFunc) volume_adjust, info);

  }


  /***************************************/
  /* Create the simultaneous button *****/
  /*  if(chan_info->caps & SND_MIXER_CINFO_CAP_STEREO) {*/
    button = gtk_toggle_button_new_with_label("|-|");
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    /* Set up the callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_SIMULTANEOUS;
    gtk_signal_connect(GTK_OBJECT (button), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
    /* And let's not forget to set the simultaneous flag */
    schan->flags |= CHANNEL_SIMULTANEOUS;
    /*  }*/
  /* If there is no simultaneous button, create a placeholder widget for asthetic purposes */
  /*  else {
    button = gtk_label_new("");
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);
    }*/

  /*********************************/
  /* Create the Record button *****/
  if(chan_info->caps & SND_MIXER_CINFO_CAP_RECORD) {
    button = gtk_toggle_button_new_with_label("Rec");
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);
    /* Store the widget in the channel struct */
    schan->rec = button;
    
    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_RECORD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);

    /* Set up the callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_RECORD;
    gtk_signal_connect(GTK_OBJECT (button), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
  }




  return box;
}
