/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

GtkWidget *window;
Card *card; /* And array of the cards */
int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/

int main(int argc, char **argv)
{
  /* Begin Variable Declarations */
  GtkWidget *mainbox;
  GtkWidget *notebook;
  GtkWidget *frame;
  GtkWidget *label;
  int i,j;
  char title[32];
  char name[32];
  /* End Variable Declarations */

  /* Go through gtk initialization */
  gtk_init(&argc, &argv);

  /* Read in the soundcard info */
  if(!init_cards()) {
    printf("Error.  Unable to initialize sound cards.\n");
    return 1;
  }

  /* Make the title */
  sprintf(title, "XAmixer %s", VERSION);

  /* Create the main window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), title);
  gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
		     (GtkSignalFunc) gtk_main_quit, NULL);
  signal(SIGINT, (void *)gtk_main_quit);

  /* Create the container box */
  mainbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER (window), mainbox);

  /* Create the notebook */
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  gtk_box_pack_start(GTK_BOX (mainbox), notebook, TRUE, TRUE, 0);
  gtk_widget_show(notebook);

  
  /* Create the notebook pages */
  for(i = 0; i < cards; i++) {
    frame = gtk_frame_new("");
    gtk_widget_show(frame);

    mainbox = gtk_hbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER (frame), mainbox);
    gtk_widget_show(mainbox);


    for(j = 0; j < card[i].cnum; j++) {
      if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_MASTER))
	sprintf(name, "Master"); 
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_BASS))
	sprintf(name, "Bass");
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_LINE))
	sprintf(name, "LineIn");
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_GAIN))
	sprintf(name, "Gain");
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_IGAIN))
	sprintf(name, "IGain");
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_OGAIN))
	sprintf(name, "OGain");
      else if(is_same(card[i].channel[j].info.name, SND_MIXER_ID_SPEAKER))
	sprintf(name, "Spkr");
      else
	sprintf(name, card[i].channel[j].info.name);
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control(name, 
					       card[i].channel[j].data.left,
					       card[i].channel[j].data.right,
					       &card[i],
					       j,
					       &card[i].channel[j].data,
					       &card[i].channel[j].info,
					       &card[i].channel[j]),
			 TRUE, TRUE, 0);

    }



    /* Create the label and add the page to the notebook */
    label = gtk_label_new(card[i].name);
    gtk_widget_show(label);
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

  }


  /* Set the up select() on the file descriptors to update upon changes */
  for (i = 0; i < cards; i++)
    gdk_input_add(snd_mixer_file_descriptor(card[i].handle),
		  GDK_INPUT_READ,
		  something_changed,
		  &card[i]);


  /* Show the window */
  gtk_widget_show(window);

  /* And go to the main event loop */
  gtk_main();




  return 0;
}

















